@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The update log API of Tokyo Tyrant
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2010 Mikio Hirabayashi
  * This file is part of Tokyo Tyrant.
  * Tokyo Tyrant is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -19,10 +19,19 @@
 #include "myconf.h"
 
 #define TCULAIOCBNUM   64                // number of AIO tasks
+#define TCULTMDEVALW   30.0              // allowed time deviance
+#define TCREPLTIMEO    60.0              // timeout of the replication socket
+
+typedef struct {                         // type of structure for a putshl operand
+  const char *vbuf;                      // region of the value.
+  int vsiz;                              // size of the region
+  int width;                             // the width of the record
+} PUTSHLOP;
 
 
 /* private function prototypes */
 static bool tculogflushaiocbp(struct aiocb *aiocbp);
+static void *tculogadbputshlproc(const void *vbuf, int vsiz, int *sp, PUTSHLOP *op);
 
 
 
@@ -69,16 +78,17 @@ void tculogdel(TCULOG *ulog){
 
 /* Set AIO control of an update log object. */
 bool tculogsetaio(TCULOG *ulog){
-  assert(ulog);
 #if defined(_SYS_LINUX_)
+  assert(ulog);
   if(ulog->base || ulog->aiocbs) return false;
-  struct aiocb *aiocbs = tcmalloc(TCULAIOCBNUM * sizeof(*aiocbs));
+  struct aiocb *aiocbs = tcmalloc(sizeof(*aiocbs) * TCULAIOCBNUM);
   for(int i = 0; i < TCULAIOCBNUM; i++){
     memset(aiocbs + i, 0, sizeof(*aiocbs));
   }
   ulog->aiocbs = aiocbs;
   return true;
 #else
+  assert(ulog);
   return true;
 #endif
 }
@@ -97,7 +107,7 @@ bool tculogopen(TCULOG *ulog, const char *base, uint64_t limsiz){
   for(int i = 0; i < ln; i++){
     const char *name = tclistval2(names, i);
     if(!tcstrbwm(name, TCULSUFFIX)) continue;
-    int id = atoi(name);
+    int id = tcatoi(name);
     char *path = tcsprintf("%s/%08d%s", base, id, TCULSUFFIX);
     if(stat(path, &sbuf) == 0 && S_ISREG(sbuf.st_mode) && id > max) max = id;
     tcfree(path);
@@ -188,7 +198,8 @@ bool tculogend(TCULOG *ulog, int idx){
 
 
 /* Write a message into an update log object. */
-bool tculogwrite(TCULOG *ulog, uint64_t ts, uint32_t sid, const void *ptr, int size){
+bool tculogwrite(TCULOG *ulog, uint64_t ts, uint32_t sid, uint32_t mid,
+                 const void *ptr, int size){
   assert(ulog && ptr && size >= 0);
   if(!ulog->base) return false;
   if(ts < 1) ts = (uint64_t)(tctime() * 1000000);
@@ -216,10 +227,13 @@ bool tculogwrite(TCULOG *ulog, uint64_t ts, uint32_t sid, const void *ptr, int s
   uint64_t llnum = TTHTONLL(ts);
   memcpy(wp, &llnum, sizeof(llnum));
   wp += sizeof(llnum);
-  uint32_t lnum = TTHTONL(sid);
-  memcpy(wp, &lnum, sizeof(lnum));
-  wp += sizeof(lnum);
-  lnum = TTHTONL(size);
+  uint16_t snum = TTHTONS(sid);
+  memcpy(wp, &snum, sizeof(snum));
+  wp += sizeof(snum);
+  snum = TTHTONS(mid);
+  memcpy(wp, &snum, sizeof(snum));
+  wp += sizeof(snum);
+  uint32_t lnum = TTHTONL(size);
   memcpy(wp, &lnum, sizeof(lnum));
   wp += sizeof(lnum);
   memcpy(wp, ptr, size);
@@ -306,7 +320,7 @@ TCULRD *tculrdnew(TCULOG *ulog, uint64_t ts){
   for(int i = 0; i < ln; i++){
     const char *name = tclistval2(names, i);
     if(!tcstrbwm(name, TCULSUFFIX)) continue;
-    int id = atoi(name);
+    int id = tcatoi(name);
     char *path = tcsprintf("%s/%08d%s", ulog->base, id, TCULSUFFIX);
     struct stat sbuf;
     if(stat(path, &sbuf) == 0 && S_ISREG(sbuf.st_mode) && id > max) max = id;
@@ -314,6 +328,7 @@ TCULRD *tculrdnew(TCULOG *ulog, uint64_t ts){
   }
   tclistdel(names);
   if(max < 1) max = 1;
+  uint64_t bts = (ts > TCULTMDEVALW * 1000000) ? ts - TCULTMDEVALW * 1000000 : 0;
   int num = 0;
   for(int i = max; i > 0; i--){
     char *path = tcsprintf("%s/%08d%s", ulog->base, i, TCULSUFFIX);
@@ -328,10 +343,8 @@ TCULRD *tculrdnew(TCULOG *ulog, uint64_t ts){
       fts = TTNTOHLL(fts);
     }
     close(fd);
-    if(ts >= fts){
-      num = i;
-      break;
-    }
+    num = i;
+    if(bts >= fts) break;
   }
   if(num < 1) num = 1;
   TCULRD *urld = tcmalloc(sizeof(*urld));
@@ -379,8 +392,8 @@ void tculrdwait(TCULRD *ulrd){
 
 
 /* Read a message from a log reader object. */
-const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp){
-  assert(ulrd && sp && tsp && sidp);
+const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp, uint32_t *midp){
+  assert(ulrd && sp && tsp && sidp && midp);
   TCULOG *ulog = ulrd->ulog;
   if(pthread_rwlock_rdlock(&ulog->rwlck) != 0) return NULL;
   if(ulrd->fd == -1){
@@ -395,7 +408,7 @@ const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp){
   int rsiz = sizeof(uint8_t) + sizeof(uint64_t) + sizeof(uint32_t) * 2;
   unsigned char buf[rsiz];
   uint64_t ts;
-  uint32_t sid, size;
+  uint32_t sid, mid, size;
   while(true){
     if(ulog->aiocbs && ulrd->num == ulog->max){
       struct stat sbuf;
@@ -430,9 +443,13 @@ const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp){
     memcpy(&ts, rp, sizeof(ts));
     ts = TTNTOHLL(ts);
     rp += sizeof(ts);
-    memcpy(&sid, rp, sizeof(sid));
-    sid = TTNTOHL(sid);
-    rp += sizeof(sid);
+    uint16_t snum;
+    memcpy(&snum, rp, sizeof(snum));
+    sid = TTNTOHS(snum);
+    rp += sizeof(snum);
+    memcpy(&snum, rp, sizeof(snum));
+    mid = TTNTOHS(snum);
+    rp += sizeof(snum);
     memcpy(&size, rp, sizeof(size));
     size = TTNTOHL(size);
     rp += sizeof(size);
@@ -450,6 +467,7 @@ const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp){
   *sp = size;
   *tsp = ts;
   *sidp = sid;
+  *midp = mid;
   ulrd->rbuf[size] = '\0';
   pthread_rwlock_unlock(&ulog->rwlck);
   return ulrd->rbuf;
@@ -457,7 +475,7 @@ const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp){
 
 
 /* Store a record into an abstract database object. */
-bool tculogadbput(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbput(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                   const void *kbuf, int ksiz, const void *vbuf, int vsiz){
   assert(ulog && adb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   bool err = false;
@@ -483,7 +501,7 @@ bool tculogadbput(TCULOG *ulog, uint32_t sid, TCADB *adb,
     memcpy(wp, vbuf, vsiz);
     wp += vsiz;
     *(wp++) = err ? 1 : 0;
-    if(!tculogwrite(ulog, 0, sid, mbuf, msiz)) err = true;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
     if(mbuf != mstack) tcfree(mbuf);
     tculogend(ulog, rmidx);
   }
@@ -492,7 +510,7 @@ bool tculogadbput(TCULOG *ulog, uint32_t sid, TCADB *adb,
 
 
 /* Store a new record into an abstract database object. */
-bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                       const void *kbuf, int ksiz, const void *vbuf, int vsiz){
   assert(ulog && adb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   bool err = false;
@@ -518,7 +536,7 @@ bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, TCADB *adb,
     memcpy(wp, vbuf, vsiz);
     wp += vsiz;
     *(wp++) = err ? 1 : 0;
-    if(!tculogwrite(ulog, 0, sid, mbuf, msiz)) err = true;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
     if(mbuf != mstack) tcfree(mbuf);
     tculogend(ulog, rmidx);
   }
@@ -527,7 +545,7 @@ bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, TCADB *adb,
 
 
 /* Concatenate a value at the end of the existing record in an abstract database object. */
-bool tculogadbputcat(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbputcat(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                      const void *kbuf, int ksiz, const void *vbuf, int vsiz){
   assert(ulog && adb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0);
   bool err = false;
@@ -553,7 +571,50 @@ bool tculogadbputcat(TCULOG *ulog, uint32_t sid, TCADB *adb,
     memcpy(wp, vbuf, vsiz);
     wp += vsiz;
     *(wp++) = err ? 1 : 0;
-    if(!tculogwrite(ulog, 0, sid, mbuf, msiz)) err = true;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
+    if(mbuf != mstack) tcfree(mbuf);
+    tculogend(ulog, rmidx);
+  }
+  return !err;
+}
+
+
+/* Concatenate a value at the end of the existing record and shift it to the left. */
+bool tculogadbputshl(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                     const void *kbuf, int ksiz, const void *vbuf, int vsiz, int width){
+  assert(ulog && adb && kbuf && ksiz >= 0 && vbuf && vsiz >= 0 && width >= 0);
+  bool err = false;
+  int rmidx = tculogrmtxidx(ulog, kbuf, ksiz);
+  bool dolog = tculogbegin(ulog, rmidx);
+  PUTSHLOP op;
+  op.vbuf = vbuf;
+  op.vsiz = vsiz;
+  op.width = width;
+  if(!tcadbputproc(adb, kbuf, ksiz, vbuf, vsiz, (TCPDPROC)tculogadbputshlproc, &op))
+    err = true;
+  if(dolog){
+    unsigned char mstack[TTIOBUFSIZ];
+    int msiz = sizeof(uint8_t) * 3 + sizeof(uint32_t) * 3 + ksiz + vsiz;
+    unsigned char *mbuf = (msiz < TTIOBUFSIZ) ? mstack : tcmalloc(msiz + 1);
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDPUTSHL;
+    uint32_t lnum;
+    lnum = TTHTONL(ksiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    lnum = TTHTONL(vsiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    lnum = TTHTONL(width);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    memcpy(wp, kbuf, ksiz);
+    wp += ksiz;
+    memcpy(wp, vbuf, vsiz);
+    wp += vsiz;
+    *(wp++) = err ? 1 : 0;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
     if(mbuf != mstack) tcfree(mbuf);
     tculogend(ulog, rmidx);
   }
@@ -562,7 +623,8 @@ bool tculogadbputcat(TCULOG *ulog, uint32_t sid, TCADB *adb,
 
 
 /* Remove a record of an abstract database object. */
-bool tculogadbout(TCULOG *ulog, uint32_t sid, TCADB *adb, const void *kbuf, int ksiz){
+bool tculogadbout(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                  const void *kbuf, int ksiz){
   assert(ulog && adb && kbuf && ksiz >= 0);
   bool err = false;
   int rmidx = tculogrmtxidx(ulog, kbuf, ksiz);
@@ -582,7 +644,7 @@ bool tculogadbout(TCULOG *ulog, uint32_t sid, TCADB *adb, const void *kbuf, int
     memcpy(wp, kbuf, ksiz);
     wp += ksiz;
     *(wp++) = err ? 1 : 0;
-    if(!tculogwrite(ulog, 0, sid, mbuf, msiz)) err = true;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
     if(mbuf != mstack) tcfree(mbuf);
     tculogend(ulog, rmidx);
   }
@@ -590,8 +652,119 @@ bool tculogadbout(TCULOG *ulog, uint32_t sid, TCADB *adb, const void *kbuf, int
 }
 
 
+/* Add an integer to a record in an abstract database object. */
+int tculogadbaddint(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                    const void *kbuf, int ksiz, int num){
+  assert(ulog && adb && kbuf && ksiz >= 0);
+  int rmidx = tculogrmtxidx(ulog, kbuf, ksiz);
+  bool dolog = num != 0 && tculogbegin(ulog, rmidx);
+  int rnum = tcadbaddint(adb, kbuf, ksiz, num);
+  if(dolog){
+    unsigned char mstack[TTIOBUFSIZ];
+    int msiz = sizeof(uint8_t) * 3 + sizeof(uint32_t) * 2 + ksiz;
+    unsigned char *mbuf = (msiz < TTIOBUFSIZ) ? mstack : tcmalloc(msiz + 1);
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDADDINT;
+    uint32_t lnum;
+    lnum = TTHTONL(ksiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    lnum = TTHTONL(num);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    memcpy(wp, kbuf, ksiz);
+    wp += ksiz;
+    *(wp++) = (rnum == INT_MIN) ? 1 : 0;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) rnum = INT_MIN;
+    if(mbuf != mstack) tcfree(mbuf);
+    tculogend(ulog, rmidx);
+  }
+  return rnum;
+}
+
+
+/* Add a real number to a record in an abstract database object. */
+double tculogadbadddouble(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                          const void *kbuf, int ksiz, double num){
+  assert(ulog && adb && kbuf && ksiz >= 0);
+  int rmidx = tculogrmtxidx(ulog, kbuf, ksiz);
+  bool dolog = num != 0 && tculogbegin(ulog, rmidx);
+  double rnum = tcadbadddouble(adb, kbuf, ksiz, num);
+  if(dolog){
+    unsigned char mstack[TTIOBUFSIZ];
+    int msiz = sizeof(uint8_t) * 3 + sizeof(uint32_t) + sizeof(uint64_t) * 2 + ksiz;
+    unsigned char *mbuf = (msiz < TTIOBUFSIZ) ? mstack : tcmalloc(msiz + 1);
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDADDDOUBLE;
+    uint32_t lnum;
+    lnum = TTHTONL(ksiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    ttpackdouble(num, (char *)wp);
+    wp += sizeof(uint64_t) * 2;
+    memcpy(wp, kbuf, ksiz);
+    wp += ksiz;
+    *(wp++) = isnan(rnum) ? 1 : 0;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) rnum = INT_MIN;
+    if(mbuf != mstack) tcfree(mbuf);
+    tculogend(ulog, rmidx);
+  }
+  return rnum;
+}
+
+
+/* Synchronize updated contents of an abstract database object with the file and the device. */
+bool tculogadbsync(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb){
+  assert(ulog && adb);
+  bool err = false;
+  bool dolog = tculogbegin(ulog, -1);
+  if(!tcadbsync(adb)) err = true;
+  if(dolog){
+    unsigned char mbuf[sizeof(uint8_t)*3];
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDSYNC;
+    *(wp++) = err ? 1 : 0;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, wp - mbuf)) err = true;
+    tculogend(ulog, -1);
+  }
+  return !err;
+}
+
+
+/* Optimize the storage of an abstract database object. */
+bool tculogadboptimize(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb, const char *params){
+  assert(ulog && adb);
+  bool err = false;
+  bool dolog = tculogbegin(ulog, -1);
+  if(!tcadboptimize(adb, params)) err = true;
+  if(dolog){
+    int psiz = strlen(params);
+    unsigned char mstack[TTIOBUFSIZ];
+    int msiz = sizeof(uint8_t) * 3 + sizeof(uint32_t) + psiz;
+    unsigned char *mbuf = (msiz < TTIOBUFSIZ) ? mstack : tcmalloc(msiz + 1);
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDOPTIMIZE;
+    uint32_t lnum;
+    lnum = TTHTONL(psiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    memcpy(wp, params, psiz);
+    wp += psiz;
+    *(wp++) = err ? 1 : 0;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)) err = true;
+    if(mbuf != mstack) tcfree(mbuf);
+    tculogend(ulog, -1);
+  }
+  return !err;
+}
+
+
 /* Remove all records of an abstract database object. */
-bool tculogadbvanish(TCULOG *ulog, uint32_t sid, TCADB *adb){
+bool tculogadbvanish(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb){
   assert(ulog && adb);
   bool err = false;
   bool dolog = tculogbegin(ulog, -1);
@@ -602,13 +775,63 @@ bool tculogadbvanish(TCULOG *ulog, uint32_t sid, TCADB *adb){
     *(wp++) = TTMAGICNUM;
     *(wp++) = TTCMDVANISH;
     *(wp++) = err ? 1 : 0;
-    if(!tculogwrite(ulog, 0, sid, mbuf, wp - mbuf)) err = true;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, wp - mbuf)) err = true;
     tculogend(ulog, -1);
   }
   return !err;
 }
 
 
+/* Call a versatile function for miscellaneous operations of an abstract database object. */
+TCLIST *tculogadbmisc(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                      const char *name, const TCLIST *args){
+  assert(ulog && adb && name && args);
+  bool dolog = tculogbegin(ulog, -1);
+  TCLIST *rv = tcadbmisc(adb, name, args);
+  if(dolog){
+    int nsiz = strlen(name);
+    int anum = tclistnum(args);
+    int msiz = sizeof(uint8_t) * 3 + sizeof(uint32_t) * 2 + nsiz;
+    for(int i = 0; i < anum; i++){
+      int esiz;
+      tclistval(args, i, &esiz);
+      msiz += sizeof(uint32_t) + esiz;
+    }
+    unsigned char mstack[TTIOBUFSIZ];
+    unsigned char *mbuf = (msiz < TTIOBUFSIZ) ? mstack : tcmalloc(msiz + 1);
+    unsigned char *wp = mbuf;
+    *(wp++) = TTMAGICNUM;
+    *(wp++) = TTCMDMISC;
+    uint32_t lnum;
+    lnum = TTHTONL(nsiz);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    lnum = TTHTONL(anum);
+    memcpy(wp, &lnum, sizeof(lnum));
+    wp += sizeof(lnum);
+    memcpy(wp, name, nsiz);
+    wp += nsiz;
+    for(int i = 0; i < anum; i++){
+      int esiz;
+      const char *ebuf = tclistval(args, i, &esiz);
+      lnum = TTHTONL(esiz);
+      memcpy(wp, &lnum, sizeof(lnum));
+      wp += sizeof(lnum);
+      memcpy(wp, ebuf, esiz);
+      wp += esiz;
+    }
+    *(wp++) = rv ? 0 : 1;
+    if(!tculogwrite(ulog, 0, sid, mid, mbuf, msiz)){
+      if(rv) tclistdel(rv);
+      rv = NULL;
+    }
+    if(mbuf != mstack) tcfree(mbuf);
+    tculogend(ulog, -1);
+  }
+  return rv;
+}
+
+
 /* Restore an abstract database object. */
 bool tculogadbrestore(TCADB *adb, const char *path, uint64_t ts, bool con, TCULOG *ulog){
   assert(adb && path);
@@ -620,9 +843,10 @@ bool tculogadbrestore(TCADB *adb, const char *path, uint64_t ts, bool con, TCULO
       const char *rbuf;
       int rsiz;
       uint64_t rts;
-      uint32_t rsid;
-      while((rbuf = tculrdread(ulrd, &rsiz, &rts, &rsid)) != NULL){
-        if(!tculogadbredo(adb, rbuf, rsiz, con, ulog, rsid)){
+      uint32_t rsid, rmid;
+      while((rbuf = tculrdread(ulrd, &rsiz, &rts, &rsid, &rmid)) != NULL){
+        bool cc;
+        if(!tculogadbredo(adb, rbuf, rsiz, ulog, rsid, rmid, &cc) || (con && !cc)){
           err = true;
           break;
         }
@@ -641,7 +865,8 @@ bool tculogadbrestore(TCADB *adb, const char *path, uint64_t ts, bool con, TCULO
 
 
 /* Redo an update log message. */
-bool tculogadbredo(TCADB *adb, const char *ptr, int size, bool con, TCULOG *ulog, uint32_t sid){
+bool tculogadbredo(TCADB *adb, const char *ptr, int size, TCULOG *ulog,
+                   uint32_t sid, uint32_t mid, bool *cp){
   assert(adb && ptr && size >= 0);
   if(size < sizeof(uint8_t) * 3) return false;
   const unsigned char *rp = (unsigned char *)ptr;
@@ -651,79 +876,184 @@ bool tculogadbredo(TCADB *adb, const char *ptr, int size, bool con, TCULOG *ulog
   size -= sizeof(uint8_t) * 3;
   if(magic != TTMAGICNUM) return false;
   bool err = false;
+  *cp = true;
   switch(cmd){
-  case TTCMDPUT:
-    if(size >= sizeof(uint32_t) * 2){
-      uint32_t ksiz;
-      memcpy(&ksiz, rp, sizeof(ksiz));
-      ksiz = TTNTOHL(ksiz);
-      rp += sizeof(ksiz);
-      uint32_t vsiz;
-      memcpy(&vsiz, rp, sizeof(vsiz));
-      vsiz = TTNTOHL(vsiz);
-      rp += sizeof(vsiz);
-      if(tculogadbput(ulog, sid, adb, rp, ksiz, rp + ksiz, vsiz) != exp && con) err = true;
-    } else {
+    case TTCMDPUT:
+      if(size >= sizeof(uint32_t) * 2){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        uint32_t vsiz;
+        memcpy(&vsiz, rp, sizeof(vsiz));
+        vsiz = TTNTOHL(vsiz);
+        rp += sizeof(vsiz);
+        if(tculogadbput(ulog, sid, mid, adb, rp, ksiz, rp + ksiz, vsiz) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDPUTKEEP:
+      if(size >= sizeof(uint32_t) * 2){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        uint32_t vsiz;
+        memcpy(&vsiz, rp, sizeof(vsiz));
+        vsiz = TTNTOHL(vsiz);
+        rp += sizeof(vsiz);
+        if(tculogadbputkeep(ulog, sid, mid, adb, rp, ksiz, rp + ksiz, vsiz) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDPUTCAT:
+      if(size >= sizeof(uint32_t) * 2){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        uint32_t vsiz;
+        memcpy(&vsiz, rp, sizeof(vsiz));
+        vsiz = TTNTOHL(vsiz);
+        rp += sizeof(vsiz);
+        if(tculogadbputcat(ulog, sid, mid, adb, rp, ksiz, rp + ksiz, vsiz) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDPUTSHL:
+      if(size >= sizeof(uint32_t) * 3){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        uint32_t vsiz;
+        memcpy(&vsiz, rp, sizeof(vsiz));
+        vsiz = TTNTOHL(vsiz);
+        rp += sizeof(vsiz);
+        uint32_t width;
+        memcpy(&width, rp, sizeof(width));
+        width = TTNTOHL(width);
+        rp += sizeof(width);
+        if(tculogadbputshl(ulog, sid, mid, adb, rp, ksiz, rp + ksiz, vsiz, width) != exp)
+          *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDOUT:
+      if(size >= sizeof(uint32_t)){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        if(tculogadbout(ulog, sid, mid, adb, rp, ksiz) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDADDINT:
+      if(size >= sizeof(uint32_t) * 2){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        int32_t num;
+        memcpy(&num, rp, sizeof(num));
+        num = TTNTOHL(num);
+        rp += sizeof(num);
+        int rnum = tculogadbaddint(ulog, sid, mid, adb, rp, ksiz, num);
+        if(exp && rnum == INT_MIN) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDADDDOUBLE:
+      if(size >= sizeof(uint32_t) + sizeof(uint64_t) * 2){
+        uint32_t ksiz;
+        memcpy(&ksiz, rp, sizeof(ksiz));
+        ksiz = TTNTOHL(ksiz);
+        rp += sizeof(ksiz);
+        double num = ttunpackdouble((char *)rp);
+        rp += sizeof(uint64_t) * 2;
+        double rnum = tculogadbadddouble(ulog, sid, mid, adb, rp, ksiz, num);
+        if(exp && isnan(rnum)) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDSYNC:
+      if(size == 0){
+        if(tculogadbsync(ulog, sid, mid, adb) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDOPTIMIZE:
+      if(size >= sizeof(uint32_t)){
+        uint32_t psiz;
+        memcpy(&psiz, rp, sizeof(psiz));
+        psiz = TTNTOHL(psiz);
+        rp += sizeof(psiz);
+        char *params = tcmemdup(rp, psiz);
+        if(tculogadboptimize(ulog, sid, mid, adb, params) != exp) *cp = false;
+        tcfree(params);
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDVANISH:
+      if(size == 0){
+        if(tculogadbvanish(ulog, sid, mid, adb) != exp) *cp = false;
+      } else {
+        err = true;
+      }
+      break;
+    case TTCMDMISC:
+      if(size >= sizeof(uint32_t) * 2){
+        uint32_t nsiz;
+        memcpy(&nsiz, rp, sizeof(nsiz));
+        nsiz = TTNTOHL(nsiz);
+        rp += sizeof(nsiz);
+        uint32_t anum;
+        memcpy(&anum, rp, sizeof(anum));
+        anum = TTNTOHL(anum);
+        rp += sizeof(anum);
+        char *name = tcmemdup(rp, nsiz);
+        rp += nsiz;
+        TCLIST *args = tclistnew2(anum);
+        for(int i = 0; i < anum; i++){
+          uint32_t esiz;
+          memcpy(&esiz, rp, sizeof(esiz));
+          esiz = TTNTOHL(esiz);
+          rp += sizeof(esiz);
+          tclistpush(args, rp, esiz);
+          rp += esiz;
+        }
+        TCLIST *res = tculogadbmisc(ulog, sid, mid, adb, name, args);
+        if(res){
+          if(!exp) *cp = false;
+          tclistdel(res);
+        } else {
+          if(exp) *cp = false;
+        }
+        tclistdel(args);
+        tcfree(name);
+      } else {
+        err = true;
+      }
+      break;
+    default:
       err = true;
-    }
-    break;
-  case TTCMDPUTKEEP:
-    if(size >= sizeof(uint32_t) * 2){
-      uint32_t ksiz;
-      memcpy(&ksiz, rp, sizeof(ksiz));
-      ksiz = TTNTOHL(ksiz);
-      rp += sizeof(ksiz);
-      uint32_t vsiz;
-      memcpy(&vsiz, rp, sizeof(vsiz));
-      vsiz = TTNTOHL(vsiz);
-      rp += sizeof(vsiz);
-      if(tculogadbputkeep(ulog, sid, adb, rp, ksiz, rp + ksiz, vsiz) != exp && con) err = true;
-    } else {
-      err = true;
-    }
-    break;
-  case TTCMDPUTCAT:
-    if(size >= sizeof(uint32_t) * 2){
-      uint32_t ksiz;
-      memcpy(&ksiz, rp, sizeof(ksiz));
-      ksiz = TTNTOHL(ksiz);
-      rp += sizeof(ksiz);
-      uint32_t vsiz;
-      memcpy(&vsiz, rp, sizeof(vsiz));
-      vsiz = TTNTOHL(vsiz);
-      rp += sizeof(vsiz);
-      if(tculogadbputcat(ulog, sid, adb, rp, ksiz, rp + ksiz, vsiz) != exp && con) err = true;
-    } else {
-      err = true;
-    }
-    break;
-  case TTCMDOUT:
-    if(size >= sizeof(uint32_t)){
-      uint32_t ksiz;
-      memcpy(&ksiz, rp, sizeof(ksiz));
-      ksiz = TTNTOHL(ksiz);
-      rp += sizeof(ksiz);
-      if(tculogadbout(ulog, sid, adb, rp, ksiz) != exp && con) err = true;
-    } else {
-      err = true;
-    }
-    break;
-  case TTCMDVANISH:
-    if(size == 0){
-      if(tculogadbvanish(ulog, sid, adb) != exp && con) err = true;
-    } else {
-      err = true;
-    }
-    break;
-  default:
-    err = true;
-    break;
+      break;
   }
   return !err;
 }
 
 
-/* Create a replicatoin object. */
+/* Create a replication object. */
 TCREPL *tcreplnew(void){
   TCREPL *repl = tcmalloc(sizeof(*repl));
   repl->fd = -1;
@@ -744,6 +1074,8 @@ void tcrepldel(TCREPL *repl){
 bool tcreplopen(TCREPL *repl, const char *host, int port, uint64_t ts, uint32_t sid){
   assert(repl && host && port >= 0);
   if(repl->fd >= 0) return false;
+  if(ts < 1) ts = 1;
+  if(sid < 1) sid = INT_MAX;
   char addr[TTADDRBUFSIZ];
   if(!ttgethostaddr(host, addr)) return false;
   int fd = ttopensock(addr, port);
@@ -755,7 +1087,7 @@ bool tcreplopen(TCREPL *repl, const char *host, int port, uint64_t ts, uint32_t
   uint64_t llnum = TTHTONLL(ts);
   memcpy(wp, &llnum, sizeof(llnum));
   wp += sizeof(llnum);
-  uint64_t lnum = TTHTONL(sid);
+  uint32_t lnum = TTHTONL(sid);
   memcpy(wp, &lnum, sizeof(lnum));
   wp += sizeof(lnum);
   repl->fd = fd;
@@ -766,6 +1098,11 @@ bool tcreplopen(TCREPL *repl, const char *host, int port, uint64_t ts, uint32_t
     tcreplclose(repl);
     return false;
   }
+  repl->mid = ttsockgetint32(repl->sock);
+  if(ttsockcheckend(repl->sock) || repl->mid < 1){
+    tcreplclose(repl);
+    return false;
+  }
   return true;
 }
 
@@ -789,6 +1126,7 @@ const char *tcreplread(TCREPL *repl, int *sp, uint64_t *tsp, uint32_t *sidp){
   assert(repl && sp && tsp);
   int ocs = PTHREAD_CANCEL_DISABLE;
   pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &ocs);
+  ttsocksetlife(repl->sock, TCREPLTIMEO);
   int c = ttsockgetc(repl->sock);
   if(c == TCULMAGICNOP){
     *sp = 0;
@@ -843,5 +1181,31 @@ static bool tculogflushaiocbp(struct aiocb *aiocbp){
 }
 
 
+/* Call back function for the putshl function.
+   `vbuf' specifies the pointer to the region of the value.
+   `vsiz' specifies the size of the region of the value.
+   `sp' specifies the pointer to the variable into which the size of the region of the return
+   value is assigned.
+   `op' specifies the pointer to the optional opaque object.
+   The return value is the pointer to the result object. */
+static void *tculogadbputshlproc(const void *vbuf, int vsiz, int *sp, PUTSHLOP *op){
+  assert(vbuf && vsiz >= 0 && sp && op);
+  int rsiz = tclmin(vsiz + op->vsiz, op->width);
+  char *rbuf = tcmalloc(rsiz + 1);
+  char *wp = rbuf;
+  int wsiz = rsiz;
+
+  int left = wsiz - op->vsiz;
+  if(left > 0){
+    memcpy(wp, (char *)vbuf + vsiz - left, left);
+    wp += left;
+    wsiz -= left;
+  }
+  if(wsiz > 0) memcpy(wp, op->vbuf + op->vsiz - wsiz, wsiz);
+  *sp = rsiz;
+  return rbuf;
+}
+
+
 
 // END OF FILE
