@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The update log API of Tokyo Tyrant
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2010 Mikio Hirabayashi
  * This file is part of Tokyo Tyrant.
  * Tokyo Tyrant is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -27,14 +27,7 @@
 __TCULOG_CLINKAGEBEGIN
 
 
-#include <tcutil.h>
-#include <tcadb.h>
 #include <ttutil.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
-#include <pthread.h>
 
 
 
@@ -77,6 +70,7 @@ typedef struct {                         /* type of structure for a replication
   TTSOCK *sock;                          /* socket object */
   char *rbuf;                            /* record buffer */
   int rsiz;                              /* size of the record buffer */
+  uint16_t mid;                          /* master server ID number */
 } TCREPL;
 
 
@@ -136,11 +130,13 @@ bool tculogend(TCULOG *ulog, int idx);
 /* Write a message into an update log object.
    `ulog' specifies the update log object.
    `ts' specifies the timestamp.  If it is 0, the current time is specified.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `ptr' specifies the pointer to the region of the message.
    `size' specifies the size of the region.
    If successful, the return value is true, else, it is false. */
-bool tculogwrite(TCULOG *ulog, uint64_t ts, uint32_t sid, const void *ptr, int size);
+bool tculogwrite(TCULOG *ulog, uint64_t ts, uint32_t sid, uint32_t mid,
+                 const void *ptr, int size);
 
 
 /* Create a log reader object.
@@ -166,16 +162,19 @@ void tculrdwait(TCULRD *ulrd);
    value is assigned.
    `tsp' specifies the pointer to the variable into which the timestamp of the next message is
    assigned.
-   `sidp' specifies the pointer to the variable into which the server ID of the next message is
-   assigned.
+   `sidp' specifies the pointer to the variable into which the origin server ID of the next
+   message is assigned.
+   `midp' specifies the pointer to the variable into which the master server ID of the next
+   message is assigned.
    If successful, the return value is the pointer to the region of the value of the next message.
    `NULL' is returned if no record is to be read. */
-const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp);
+const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp, uint32_t *midp);
 
 
 /* Store a record into an abstract database object.
    `ulog' specifies the update log object.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `adb' specifies the abstract database object.
    `kbuf' specifies the pointer to the region of the key.
    `ksiz' specifies the size of the region of the key.
@@ -183,13 +182,14 @@ const void *tculrdread(TCULRD *ulrd, int *sp, uint64_t *tsp, uint32_t *sidp);
    `vsiz' specifies the size of the region of the value.
    If successful, the return value is true, else, it is false.
    If a record with the same key exists in the database, it is overwritten. */
-bool tculogadbput(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbput(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                   const void *kbuf, int ksiz, const void *vbuf, int vsiz);
 
 
 /* Store a new record into an abstract database object.
    `ulog' specifies the update log object.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `adb' specifies the abstract database object.
    `kbuf' specifies the pointer to the region of the key.
    `ksiz' specifies the size of the region of the key.
@@ -197,13 +197,14 @@ bool tculogadbput(TCULOG *ulog, uint32_t sid, TCADB *adb,
    `vsiz' specifies the size of the region of the value.
    If successful, the return value is true, else, it is false.
    If a record with the same key exists in the database, this function has no effect. */
-bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                       const void *kbuf, int ksiz, const void *vbuf, int vsiz);
 
 
 /* Concatenate a value at the end of the existing record in an abstract database object.
    `ulog' specifies the update log object.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `adb' specifies the abstract database object.
    `kbuf' specifies the pointer to the region of the key.
    `ksiz' specifies the size of the region of the key.
@@ -211,25 +212,113 @@ bool tculogadbputkeep(TCULOG *ulog, uint32_t sid, TCADB *adb,
    `vsiz' specifies the size of the region of the value.
    If successful, the return value is true, else, it is false.
    If there is no corresponding record, a new record is created. */
-bool tculogadbputcat(TCULOG *ulog, uint32_t sid, TCADB *adb,
+bool tculogadbputcat(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
                      const void *kbuf, int ksiz, const void *vbuf, int vsiz);
 
 
+/* Concatenate a value at the end of the existing record and shift it to the left.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `vbuf' specifies the pointer to the region of the value.
+   `vsiz' specifies the size of the region of the value.
+   `width' specifies the width of the record.
+   If successful, the return value is true, else, it is false.
+   If there is no corresponding record, a new record is created. */
+bool tculogadbputshl(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                     const void *kbuf, int ksiz, const void *vbuf, int vsiz, int width);
+
+
 /* Remove a record of an abstract database object.
    `ulog' specifies the update log object.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `adb' specifies the abstract database object.
    `kbuf' specifies the pointer to the region of the key.
    `ksiz' specifies the size of the region of the key.
    If successful, the return value is true, else, it is false. */
-bool tculogadbout(TCULOG *ulog, uint32_t sid, TCADB *adb, const void *kbuf, int ksiz);
+bool tculogadbout(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                  const void *kbuf, int ksiz);
+
+
+/* Add an integer to a record in an abstract database object.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is `INT_MIN'.
+   If the corresponding record exists, the value is treated as an integer and is added to.  If no
+   record corresponds, a new record of the additional value is stored. */
+int tculogadbaddint(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                    const void *kbuf, int ksiz, int num);
+
+
+/* Add a real number to a record in an abstract database object.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object connected as a writer.
+   `kbuf' specifies the pointer to the region of the key.
+   `ksiz' specifies the size of the region of the key.
+   `num' specifies the additional value.
+   If successful, the return value is the summation value, else, it is `NAN'.
+   If the corresponding record exists, the value is treated as a real number and is added to.  If
+   no record corresponds, a new record of the additional value is stored. */
+double tculogadbadddouble(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                          const void *kbuf, int ksiz, double num);
+
+
+/* Synchronize updated contents of an abstract database object with the file and the device.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object.
+   If successful, the return value is true, else, it is false. */
+bool tculogadbsync(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb);
+
+
+/* Optimize the storage of an abstract database object.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object.
+   `params' specifies the string of the tuning parameters, which works as with the tuning
+   of parameters the function `tcadbopen'.  If it is `NULL', it is not used.
+   If successful, the return value is true, else, it is false. */
+bool tculogadboptimize(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb, const char *params);
 
 
 /* Remove all records of an abstract database object.
    `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
    `adb' specifies the abstract database object.
    If successful, the return value is true, else, it is false. */
-bool tculogadbvanish(TCULOG *ulog, uint32_t sid, TCADB *adb);
+bool tculogadbvanish(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb);
+
+
+/* Call a versatile function for miscellaneous operations of an abstract database object.
+   `ulog' specifies the update log object.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `adb' specifies the abstract database object.
+   `name' specifies the name of the function.
+   `args' specifies a list object containing arguments.
+   If successful, the return value is a list object of the result.  `NULL' is returned on failure.
+   All databases support "putlist", "outlist", and "getlist".  "putdup" is to store records.  It
+   receives keys and values one after the other, and returns an empty list.  "outlist" is to
+   remove records.  It receives keys, and returns an empty list.  "getlist" is to retrieve
+   records.  It receives keys, and returns values.  Because the object of the return value is
+   created with the function `tclistnew', it should be deleted with the function `tclistdel' when
+   it is no longer in use. */
+TCLIST *tculogadbmisc(TCULOG *ulog, uint32_t sid, uint32_t mid, TCADB *adb,
+                      const char *name, const TCLIST *args);
 
 
 /* Restore an abstract database object.
@@ -246,14 +335,17 @@ bool tculogadbrestore(TCADB *adb, const char *path, uint64_t ts, bool con, TCULO
    `adb' specifies the abstract database object.
    `ptr' specifies the pointer to the region of the message.
    `size' specifies the size of the region.
-   `con' specifies whether consistency checking is performed.
    `ulog' specifies the update log object.
-   `sid' specifies the server ID of the message.
+   `sid' specifies the origin server ID of the message.
+   `mid' specifies the master server ID of the message.
+   `cp' specifies the pointer to the variable into which the result of consistency checking is
+   assigned.
    If successful, the return value is true, else, it is false. */
-bool tculogadbredo(TCADB *adb, const char *ptr, int size, bool con, TCULOG *ulog, uint32_t sid);
+bool tculogadbredo(TCADB *adb, const char *ptr, int size, TCULOG *ulog,
+                   uint32_t sid, uint32_t mid, bool *cp);
 
 
-/* Create a replicatoin object.
+/* Create a replication object.
    The return value is the new replicatoin object. */
 TCREPL *tcreplnew(void);
 
@@ -284,8 +376,8 @@ bool tcreplclose(TCREPL *repl);
    value is assigned.
    `tsp' specifies the pointer to the variable into which the timestamp of the next message is
    assigned.
-   `sidp' specifies the pointer to the variable into which the server ID of the next message is
-   assigned.
+   `sidp' specifies the pointer to the variable into which the origin server ID of the next
+   message is assigned.
    If successful, the return value is the pointer to the region of the value of the next message.
    `NULL' is returned if no record is to be read.  Empty string is returned when the no-operation
    command has been received. */
