@@ -1,6 +1,6 @@
 /*************************************************************************************************
  * The utility API of Tokyo Tyrant
- *                                                      Copyright (C) 2006-2008 Mikio Hirabayashi
+ *                                                      Copyright (C) 2006-2010 Mikio Hirabayashi
  * This file is part of Tokyo Tyrant.
  * Tokyo Tyrant is free software; you can redistribute it and/or modify it under the terms of
  * the GNU Lesser General Public License as published by the Free Software Foundation; either
@@ -28,10 +28,11 @@ __TTUTIL_CLINKAGEBEGIN
 
 
 #include <tcutil.h>
-#include <stdlib.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
+#include <tchdb.h>
+#include <tcbdb.h>
+#include <tcfdb.h>
+#include <tctdb.h>
+#include <tcadb.h>
 #include <pthread.h>
 
 
@@ -50,6 +51,8 @@ typedef struct {                         /* type of structure for a socket */
   char *rp;                              /* reading pointer */
   char *ep;                              /* end pointer */
   bool end;                              /* end flag */
+  double to;                             /* timeout */
+  double dl;                             /* deadline time */
 } TTSOCK;
 
 
@@ -121,6 +124,14 @@ int ttacceptsockunix(int fd);
 bool ttclosesock(int fd);
 
 
+/* Wait an I/O event of a socket.
+   `fd' specifies the file descriptor.
+   `mode' specifies the kind of events; 0 for reading, 1 for writing, 2 for exception.
+   `timeout' specifies the timeout in seconds.
+   If successful, the return value is true, else, it is false. */
+bool ttwaitsock(int fd, int mode, double timeout);
+
+
 /* Create a socket object.
    `fd' specifies the file descriptor.
    The return value is the socket object. */
@@ -132,6 +143,12 @@ TTSOCK *ttsocknew(int fd);
 void ttsockdel(TTSOCK *sock);
 
 
+/* Set the lifetime of a socket object.
+   `sock' specifies the socket object.
+   `lifetime' specifies the lifetime seconds of each task.  By default, there is no limit. */
+void ttsocksetlife(TTSOCK *sock, double lifetime);
+
+
 /* Send data by a socket.
    `sock' specifies the socket object.
    `buf' specifies the pointer to the region of the data to send.
@@ -183,6 +200,15 @@ void ttsockungetc(TTSOCK *sock, int c);
 bool ttsockgets(TTSOCK *sock, char *buf, int size);
 
 
+/* Receive one line by a socket into allocated buffer.
+   `sock' specifies the socket object.
+   If successful, the return value is the pointer to the result buffer, else, it is `NULL'.
+   `NULL' is returned if the socket is closed before receiving linefeed.
+   Because  the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call when it is no longer in use. */
+char *ttsockgets2(TTSOCK *sock);
+
+
 /* Receive an 32-bit integer by a socket.
    `sock' specifies the socket object.
    The return value is the 32-bit integer. */
@@ -209,13 +235,13 @@ int ttsockcheckpfsiz(TTSOCK *sock);
 
 /* Fetch the resource of a URL by HTTP.
    `url' specifies the URL.
-   `reqheads' specifies a map object contains request header names and their values.  If it is
-   NULL, it is not used.
+   `reqheads' specifies a map object contains request header names and their values.  The header
+   "X-TT-Timeout" specifies the timeout in seconds.  If it is `NULL', it is not used.
    `resheads' specifies a map object to store response headers their values.  If it is NULL, it
    is not used.  Each key of the map is an uncapitalized header name.  The key "STATUS" means the
    status line.
    `resbody' specifies a extensible string object to store the entity body of the result.  If it
-   is NULL, it is not used.
+   is `NULL', it is not used.
    The return value is the response code or -1 on network error. */
 int tthttpfetch(const char *url, TCMAP *reqheads, TCMAP *resheads, TCXSTR *resbody);
 
@@ -240,11 +266,12 @@ double ttunpackdouble(const char *buf);
  *************************************************************************************************/
 
 
+#define TTDEFPORT      1978              /* default port of the server */
 #define TTMAGICNUM     0xc8              /* magic number of each command */
 #define TTCMDPUT       0x10              /* ID of put command */
 #define TTCMDPUTKEEP   0x11              /* ID of putkeep command */
 #define TTCMDPUTCAT    0x12              /* ID of putcat command */
-#define TTCMDPUTRTT    0x13              /* ID of putrtt command */
+#define TTCMDPUTSHL    0x13              /* ID of putshl command */
 #define TTCMDPUTNR     0x18              /* ID of putnr command */
 #define TTCMDOUT       0x20              /* ID of out command */
 #define TTCMDGET       0x30              /* ID of get command */
@@ -255,20 +282,28 @@ double ttunpackdouble(const char *buf);
 #define TTCMDFWMKEYS   0x58              /* ID of fwmkeys command */
 #define TTCMDADDINT    0x60              /* ID of addint command */
 #define TTCMDADDDOUBLE 0x61              /* ID of adddouble command */
+#define TTCMDEXT       0x68              /* ID of ext command */
 #define TTCMDSYNC      0x70              /* ID of sync command */
-#define TTCMDVANISH    0x71              /* ID of vanish command */
-#define TTCMDCOPY      0x72              /* ID of copy command */
-#define TTCMDRESTORE   0x73              /* ID of restore command */
+#define TTCMDOPTIMIZE  0x71              /* ID of optimize command */
+#define TTCMDVANISH    0x72              /* ID of vanish command */
+#define TTCMDCOPY      0x73              /* ID of copy command */
+#define TTCMDRESTORE   0x74              /* ID of restore command */
 #define TTCMDSETMST    0x78              /* ID of setmst command */
 #define TTCMDRNUM      0x80              /* ID of rnum command */
 #define TTCMDSIZE      0x81              /* ID of size command */
 #define TTCMDSTAT      0x88              /* ID of stat command */
+#define TTCMDMISC      0x90              /* ID of misc command */
 #define TTCMDREPL      0xa0              /* ID of repl command */
 
+#define TTTIMERMAX     8                 /* maximum number of timers */
+
 typedef struct _TTTIMER {                /* type of structure for a timer */
   pthread_t thid;                        /* thread ID */
   bool alive;                            /* alive flag */
   struct _TTSERV *serv;                  /* server object */
+  double freq_timed;                     /* frequency of timed handler */
+  void (*do_timed)(void *);              /* call back function for timed handler */
+  void *opq_timed;                       /* opaque pointer for timed handler */
 } TTTIMER;
 
 typedef struct _TTREQ {                  /* type of structure for a server */
@@ -278,6 +313,7 @@ typedef struct _TTREQ {                  /* type of structure for a server */
   int epfd;                              /* polling file descriptor */
   double mtime;                          /* last modified time */
   bool keep;                             /* keep-alive flag */
+  int idx;                               /* ordinal index */
 } TTREQ;
 
 typedef struct _TTSERV {                 /* type of structure for a server */
@@ -287,16 +323,19 @@ typedef struct _TTSERV {                 /* type of structure for a server */
   TCLIST *queue;                         /* queue of requests */
   pthread_mutex_t qmtx;                  /* mutex for the queue */
   pthread_cond_t qcnd;                   /* condition variable for the queue */
+  pthread_mutex_t tmtx;                  /* mutex for the timer */
+  pthread_cond_t tcnd;                   /* condition variable for the timer */
   int thnum;                             /* number of threads */
   double timeout;                        /* timeout milliseconds of each task */
   bool term;                             /* terminate flag */
   void (*do_log)(int, const char *, void *);  /* call back function for logging */
   void *opq_log;                         /* opaque pointer for logging */
-  double freq_timed;                     /* frequency of timed handler */
-  void (*do_timed)(void *);              /* call back function for timed handler */
-  void *opq_timed;                       /* opaque pointer for timed handler */
-  void (*do_task)(TTSOCK *, void *, TTREQ *req);  /* call back function for task */
+  TTTIMER timers[TTTIMERMAX];            /* timer objects */
+  int timernum;                          /* number of timer objects */
+  void (*do_task)(TTSOCK *, void *, TTREQ *);  /* call back function for task */
   void *opq_task;                        /* opaque pointer for task */
+  void (*do_term)(void *);               /* call back gunction for termination */
+  void *opq_term;                        /* opaque pointer for termination */
 } TTSERV;
 
 enum {                                   /* enumeration for logging levels */
@@ -344,13 +383,13 @@ void ttservtune(TTSERV *serv, int thnum, double timeout);
 void ttservsetloghandler(TTSERV *serv, void (*do_log)(int, const char *, void *), void *opq);
 
 
-/* Set the timed handler of a server object.
+/* Add a timed handler to a server object.
    `serv' specifies the server object.
    `freq' specifies the frequency of execution in seconds.
    `do_timed' specifies the pointer to a function to do with a event.  Its parameter is the
    opaque pointer.
    `opq' specifies the opaque pointer to be passed to the handler.  It can be `NULL'. */
-void ttservsettimedhandler(TTSERV *serv, double freq, void (*do_timed)(void *), void *opq);
+void ttservaddtimedhandler(TTSERV *serv, double freq, void (*do_timed)(void *), void *opq);
 
 
 /* Set the response handler of a server object.
@@ -362,6 +401,14 @@ void ttservsettimedhandler(TTSERV *serv, double freq, void (*do_timed)(void *),
 void ttservsettaskhandler(TTSERV *serv, void (*do_task)(TTSOCK *, void *, TTREQ *), void *opq);
 
 
+/* Set the termination handler of a server object.
+   `serv' specifies the server object.
+   `do_term' specifies the pointer to a function to do with a task.  Its parameter is the opaque
+   pointer.
+   `opq' specifies the opaque pointer to be passed to the handler.  It can be `NULL'. */
+void ttservsettermhandler(TTSERV *serv, void (*do_term)(void *), void *opq);
+
+
 /* Start the service of a server object.
    `serv' specifies the server object.
    If successful, the return value is true, else, it is false. */
@@ -388,15 +435,55 @@ void ttservlog(TTSERV *serv, int level, const char *format, ...);
 bool ttserviskilled(TTSERV *serv);
 
 
+/* Break a simple server expression.
+   `expr' specifies the simple server expression.  It is composed of two substrings separated
+   by ":".  The former field specifies the name or the address of the server.  The latter field
+   specifies the port number.  If the latter field is omitted, the default port number is
+   specified.
+   `pp' specifies the pointer to a variable to which the port number is assigned.  If it is
+   `NULL', it is not used.
+   The return value is the string of the host name.
+   Because  the region of the return value is allocated with the `malloc' call, it should be
+   released with the `free' call when it is no longer in use. */
+char *ttbreakservexpr(const char *expr, int *pp);
+
+
 
 /*************************************************************************************************
  * features for experts
  *************************************************************************************************/
 
 
-#define _TT_VERSION    "1.0.1"
-#define _TT_LIBVER     117
-#define _TT_PROTVER    "0.9"
+#define _TT_VERSION    "1.1.40"
+#define _TT_LIBVER     323
+#define _TT_PROTVER    "0.91"
+
+
+/* Switch the process into the background.
+   If successful, the return value is true, else, it is false. */
+bool ttdaemonize(void);
+
+
+/* Get the load average of the system.
+   The return value is the load average of the system. */
+double ttgetloadavg(void);
+
+
+/* Convert a string to a time stamp.
+   `str' specifies the string.
+   The return value is the time stamp. */
+uint64_t ttstrtots(const char *str);
+
+
+/* Get the command name of a command ID number.
+   `id' specifies the command ID number.
+   The return value is the string of the command name. */
+const char *ttcmdidtostr(int id);
+
+
+/* tricks for backward compatibility */
+#define tcrdbqrysetmax(TC_tdb, TC_max) \
+  tcrdbqrysetlimit((TC_tdb), (TC_max), 0)
 
 
 
